library ACCES32;

uses
  Windows,
  SysUtils, Classes,
  Registry,
  accesnt in 'accesnt.pas',
  acces95 in 'acces95.pas',
  WDMIO in 'WDMIO.pas',
  AIOETHW in 'AIOETHW.pas';

{$R *.RES}

const
    KEY_WOW64_64KEY = $0100;
    kWin = 1;
    kNT = 2;
    kNT_AIOWDM = 3;

var
    OS: Integer;
    ETHRef: TClientRef;
    ETHHost: AnsiString;
    IndirectAddr: DWORD;
    ReportData: record
      Reg: TRegistry;
      Index: Integer;
      {
      hMap: THandle;
      pMap: PLongWord;
      hMutex: THandle;
      }
    end;

procedure ReportIOByte(Address: Word; DirSymbol: String; DataByte: Byte; Status: LongWord);
const
  MyKey = 'Software\ACCES\OverrideDebug\';

begin
  if ReportData.Reg = nil then begin
    ReportData.Reg := TRegistry.Create(KEY_ALL_ACCESS);
    ReportData.Reg.RootKey := HKEY_CURRENT_USER;
    ReportData.Reg.DeleteKey(MyKey);
    if ReportData.Reg.OpenKey(MyKey, True) then ReportData.Reg.CloseKey;
    ReportData.Reg.Access := KEY_SET_VALUE;
    ReportData.Index := 0;
  end;

  if not ReportData.Reg.OpenKey(MyKey, False) then Exit;

  ReportData.Reg.WriteString(IntToHex(ReportData.Index, 8), IntToHex(Address, 1) + DirSymbol + IntToHex(DataByte, 2) + ' (' + IntToStr(Status) + ')');
  Inc(ReportData.Index);

  ReportData.Reg.CloseKey;
end;

function InPortB(Address: DWORD): WORD; cdecl;
var
  DataBytes: LongInt;
  Status: LongWord;

begin
    if (ETHRef <> nil) and ((Address = IndirectAddr) or (Address = IndirectAddr+1)) then begin
        DataBytes := 1;
        Result := 0;
        Status := AEW_DIO_ReadAll(ETHRef, @DataBytes, PByte(@Result));
        ReportIOByte(Address, '>>', Result, Status);
        if Status = ERROR_SUCCESS then begin
            Result := not Result;
        end
        else begin
            Result := $AA55;
            AEW_Disconnect(ETHRef);
            ETHRef := AEW_Connect(PAnsiChar(ETHHost), 0);
        end;
        SetLastError(Status);
    end
    else case OS of
        kWin: Result := Win_InPortB(Address);
        kNT: Result := NT_InPortB(Address);
        kNT_AIOWDM: Result := AIOWDM_InPortB(Address);
        else Result := $AA55;
    end;
end;

function InPort(Address: DWORD): WORD; cdecl;
begin
    case OS of
      kWin: Result := Win_InPort(Address);
      kNT: Result := NT_InPort(Address);
      kNT_AIOWDM: Result := AIOWDM_InPort(Address);
      else Result := $AA55;
    end;
end;

function InPortL(Address: DWORD): DWORD; cdecl;
begin
    case OS of
      kWin: Result := Win_InPortL(Address);
      kNT: Result := NT_InPortL(Address);
      kNT_AIOWDM: Result := AIOWDM_InPortL(Address);
      else Result := $AA55;
    end;
end;

function OutPortB(Address: DWORD; Value: BYTE): WORD; cdecl;
var
  DataBytes: LongInt;
  Status: LongWord;

begin
    if (ETHRef <> nil) and (Address = IndirectAddr) then begin
        Value := not Value;
        DataBytes := SizeOf(Value);
        Status := AEW_DIO_WriteAll(ETHRef, @DataBytes, @Value);
        ReportIOByte(Address, '<<', Value, Status);
        if Status = ERROR_SUCCESS then begin
            Result := 0;
        end
        else begin
            Result := $AA55;
            AEW_Disconnect(ETHRef);
            ETHRef := AEW_Connect(PAnsiChar(ETHHost), 0);
        end;
        SetLastError(Status);
    end
    else case OS of
        kWin: Result := Win_OutPortB(Address, Value);
        kNT: Result := NT_OutPortB(Address, Value);
        kNT_AIOWDM: Result := AIOWDM_OutPortB(Address, Value);
        else Result := $AA55;
    end;
end;

function OutPort(Address: DWORD; Value: WORD): WORD; cdecl;
begin
    case OS of
      kWin: Result := Win_OutPort(Address, Value);
      kNT: Result := NT_OutPort(Address, Value);
      kNT_AIOWDM: Result := AIOWDM_OutPort(Address, Value);
      else Result := $AA55;
    end;
end;

function OutPortL(Address: DWORD; Value: DWORD): WORD; cdecl;
begin
    case OS of
      kWin: Result := Win_OutPortL(Address, Value);
      kNT: Result := NT_OutPortL(Address, Value);
      kNT_AIOWDM: Result := AIOWDM_OutPortL(Address, Value);
      else Result := $AA55;
    end;
end;

function INSB(Port, Count: LongWord; var pBuffer:     Byte): LongWord; cdecl;
begin
  case OS of
    kWin: Result := Win_INSB(Port, Count, pBuffer);
    kNT:  Result :=  NT_INSB(Port, Count, pBuffer);
    kNT_AIOWDM: Result := ERROR_NOT_SUPPORTED;
    else  Result := ERROR_OLD_WIN_VERSION;
  end;
end;

function INSW(Port, Count: LongWord; var pBuffer:     Word): LongWord; cdecl;
begin
  case OS of
    kWin: Result := Win_INSW(Port, Count, pBuffer);
    kNT:  Result :=  NT_INSW(Port, Count, pBuffer);
    kNT_AIOWDM: Result := ERROR_NOT_SUPPORTED;
    else  Result := ERROR_OLD_WIN_VERSION;
  end;
end;

function INSD(Port, Count: LongWord; var pBuffer: LongWord): LongWord; cdecl;
begin
  case OS of
    kWin: Result := Win_INSD(Port, Count, pBuffer);
    kNT:  Result :=  NT_INSD(Port, Count, pBuffer);
    kNT_AIOWDM: Result := ERROR_NOT_SUPPORTED;
    else  Result := ERROR_OLD_WIN_VERSION;
  end;
end;



function VBInPortB(Address: DWORD): WORD; stdcall;
begin
    Result := InPortB(Address);
end;

function VBInPort(Address: DWORD): WORD; stdcall;
begin
    Result := InPort(Address);
end;

function VBInPortL(Address: DWORD): DWORD; stdcall;
begin
    Result := InPortL(Address);
end;

function VBOutPortB(Address: DWORD; Value: Byte): WORD; stdcall;
begin
    Result := OutPortB(Address, Value);
end;

function VBOutPort(Address: DWORD; Value: Word): WORD; stdcall;
begin
    Result := OutPort(Address, Value);
end;

function VBOutPortL(Address: DWORD; Value: DWord): WORD; stdcall;
begin
    Result := OutPortL(Address, Value);
end;

function VBINSB(Port, Count: LongWord; var pBuffer:     Byte): LongWord; stdcall;
begin
  Result := INSB(Port, Count, pBuffer);
end;

function VBINSW(Port, Count: LongWord; var pBuffer:     Word): LongWord; stdcall;
begin
  Result := INSW(Port, Count, pBuffer);
end;

function VBINSD(Port, Count: LongWord; var pBuffer: LongWord): LongWord; stdcall;
begin
  Result := INSD(Port, Count, pBuffer);
end;



procedure DLLHandler (Reason: Integer);
begin
    if (Reason = DLL_PROCESS_DETACH) then begin
        if (OS = kNT) then
            CloseACCESNT
        ;

        if ETHRef <> nil then begin
            AEW_Disconnect(ETHRef);
            ETHRef := nil;
        end;

        if ReportData.Reg <> nil then
          FreeAndNil(ReportData.Reg)
        ;
    end;
end;

(*exports
    InPortB,                      InPortB name '_InPortB',
    OutPortB,                     OutPortB name '_OutPortB',
    InPort,                       InPort name '_InPort',
    OutPort,                      OutPort name '_OutPort',
    InPortL,                      InPortL name '_InPortL',
    OutPortL,                     OutPortL name '_OutPortL',
    InPortL name 'InPortDWord',   InPortL name '_InPortDWord',
    OutPortL name 'OutPortDWord', OutPortL name '_OutPortDWord',
    // these make Visual Basic happy
    VBInPortB,  VBInPort,  VBInPortL,  VBInPortL name 'VBInPortDWord',
    VBOutPortB, VBOutPort, VBOutPortL, VBOutPortL name 'VBOutPortDWord';

{*)//These are index-compatible with the old ACCES32.dll
exports
  //Order is inviolate; add new entries to the beginning
    VBINSD                          ,
    INSD       name '_INSD'         ,
    INSD                            ,
    VBINSW                          ,
    INSW       name '_INSW'         ,
    INSW                            ,
    VBINSB                          ,
    INSB       name '_INSB'         ,
    INSB                            ,
    VBOutPortL name 'VBOutPortDWord',
    VBOutPortL                      ,
    VBInPortL  name 'VBInPortDWord' ,
    VBInPortL                       ,
    OutPortL   name '_OutPortDWord' ,
    OutPortL   name 'OutPortDWord'  ,
    InPortL    name '_InPortDWord'  ,
    InPortL    name 'InPortDWord'   ,
    OutPortL   name '_OutPortL'     ,
    OutPortL                        ,
    InPortL    name '_InPortL'      ,
    InPortL                         ,
    InPortB                         ,
    InPortB    name '_InPortB'      ,
    OutPortB                        ,
    OutPortB   name '_OutPortB'     ,
    InPort                          ,
    InPort     name '_InPort'       ,
    OutPort                         ,
    OutPort    name '_OutPort'      ,
    VBInPortB                       ,
    VBInPort                        ,
    VBOutPortB                      ,
    VBOutPort                       ;
{}

procedure Main;
var
    buf: array[0..255] of char;
    Reg: TRegistry;

begin
    DLLProc := @DLLHandler;

    OS := 0;
    GetEnvironmentVariable('OS', buf, 255);
    if (StrIComp(buf, 'WINDOWS_NT') = 0) then
       OS := kNT
    else begin
       GetEnvironmentVariable('windir', buf, 255);
       if (buf <> '') then
           OS := kWin
       ;
    end;
    if OS = kNT then begin
       InitACCESNT;
       if not GotACCESNT then begin
          if InitAIOWDM then
             OS := kNT_AIOWDM
          ;
       end;
    end;

    ETHRef := nil;
    ETHHost := '';
    IndirectAddr := 0;
    Reg := TRegistry.Create(STANDARD_RIGHTS_READ or KEY_QUERY_VALUE or KEY_WOW64_64KEY);
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    if Reg.OpenKey('Software\Contrec\Contcomm\Paths', False) then begin
        try
            ETHHost := Reg.ReadString('I/O Board IP Address');
        except
            ETHHost := '';
        end;
        try
            IndirectAddr := StrToInt('$' + Reg.ReadString('I/O Card Base Address'));
        except
            IndirectAddr := 0;
        end;
    end;
    Reg.Free;

    if (ETHHost = '') or (IndirectAddr = 0) then begin
        ETHHost := '';
        IndirectAddr := 0;
    end
    else begin
        ETHRef := AEW_Connect(PAnsiChar(ETHHost), 0);
    end;

    ReportData.Reg := nil;
    ReportData.Index := 0;
end;

begin
    if Assigned(DLLProc) then Exit;

    Main;
end.
